/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Event Logger.
 *
 * The Initial Developer of the Original Code is
 *   Ben Basson <contact@cusser.net>
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Basson <contact@cusser.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var eventlogger =
{
  eventPrefs: null,
  appPrefs: null,
  windows: new Array(),
  eventDoc: null,
  eventList: null,
  strings: null,
  keypressTimeout: null,
  keypressStack: "",
  keypressTarget: null,
  lastWindowOpen: null,
  currentTargets: new Array(),

  init: function () 
  {
    eventlogger.initPrefs();
    
    // window._originalWindow defined in overlay.js
    eventlogger.windows.push(window._originalWindow);

    // make sure window and pref changes are captured
    eventloggerPrefObserver.register();
    eventloggerWindowObserver.register();
    eventloggerDeleteObserver.register();
    eventloggerFileChangeObserver.register();

    window.removeEventListener("load", eventlogger.init, false);
    window.addEventListener("unload", eventlogger.uninit, false);

    if (eventlogger.appPrefs.getBoolPref("logging"))
      eventlogger.startLogging();
    eventlogger.strings = document.getElementById("eventloggerStrings");
  },
  
  initPrefs: function ()
  {
    var prefs = Components.classes['@mozilla.org/preferences-service;1']
                  .getService(Components.interfaces.nsIPrefService);
    eventlogger.eventPrefs = prefs.getBranch('eventlogger.events.');
    eventlogger.appPrefs = prefs.getBranch('eventlogger.');
  },

  uninit: function ()
  {
    for (var n in eventlogger.windows)
      eventlogger.removeEventListeners(eventlogger.windows[n]);
    
    eventloggerPrefObserver.unregister();
    eventloggerWindowObserver.unregister();
    eventloggerDeleteObserver.unregister();
    eventloggerFileChangeObserver.unregister();
  },
  
  // starts logging, resumes if previous eventDump.xml is found and populated
  startLogging: function ()
  {
    var windowList = eventlogger.windows;
    for (var n in windowList)
      eventlogger.addEventListeners(windowList[n]);
 
    eventlogger.loadTemp();
  },
  
  // loads or creates temporary file
  loadTemp: function ()
  {
    var eventDoc = eventloggerUtils.getDOMTree();
    if (!eventDoc)
      eventDoc = document.implementation.createDocument("", "", null);
    
    eventlogger.eventDoc = eventDoc;
    
    if (eventDoc.documentElement)
    {
      var eventList = eventlogger.eventList = eventDoc.documentElement;
    }
    else
    {
      var eventList = eventDoc.createElement("eventlist");
      eventlogger.eventList = eventDoc.appendChild(eventList);
    }
  },
  
  stopLogging: function ()
  {   
    var windowList = eventlogger.windows;
    for (var n in windowList)
      eventlogger.removeEventListeners(windowList[n]);
  },
  
  // grabs all eventlogger.event prefs and turns them into listeners
  addEventListeners: function (aWindow)
  {
    var eventList = eventlogger.eventPrefs.getChildList("", {});
    for (var n in eventList)
      if (eventlogger.eventPrefs.getBoolPref(eventList[n]))
        eventlogger.addEventListener(aWindow, eventList[n]);
  },
  
  removeEventListeners: function (aWindow)
  {
    var eventList = eventlogger.eventPrefs.getChildList("", {});
    for (var n in eventList)
      eventlogger.removeEventListener(aWindow, eventList[n]);
  },
  
  addEventListener: function (aWindow, aEventType)
  {
    aWindow.addEventListener(aEventType, eventlogger.handleEvent, true);  
  },

  removeEventListener: function (aWindow, aEventType)
  {
    aWindow.removeEventListener(aEventType, eventlogger.handleEvent, true);
  },
  
  // filters out events that we don't want
  // in a less than elegant way
  filterEvent: function (aEvent)
  {
    var localName = aEvent.originalTarget.localName;
    var type = aEvent.type;
    var now = new Date();
    var currTime = now.getTime();
    var lastEvent = eventlogger.eventList.lastChild;

    if (lastEvent)
    {
      var lastType = lastEvent.getAttribute("type");
      var lastTime = lastEvent.getAttribute("timeStamp");
      var similarTime = eventlogger.timeStampSimilar(currTime, lastTime);
    }
    
    if (type == "select")
    {
      if (eventlogger.timeStampSimilar(currTime,eventlogger.lastWindowOpen))
        return false;
    }
    
    if (type == "unload")
    {
      /*
        XXX workaround for bug 197709
      */
      if (eventlogger.timeStampSimilar(currTime,eventlogger.lastWindowOpen))
        return false;
   
      /*
        XXX workaround for unload firing more times than needed
            ...always end up with the final unload event
      */
      if (lastEvent && lastType == "unload" && similarTime)
      {
        eventlogger.removeEvent(lastEvent);
        return true;
      }
      else
        return true;
    }
    
    if (localName == "tabpanels" || localName == "tab" || localName == "toolbox"
      || localName == "tree")
      return false;

    if (!localName || localName == null && aEvent.originalTarget.nodeName == null)
      return false;

    if (!lastEvent)
      return true;

    if (type == "resize" && lastType == "select" && similarTime)
      return false;

    if (type == "command" && lastType == "select" && similarTime)
      return false;

    if (type == "select" && similarTime && (lastType == "select" || lastType == "click"))
    {
      var originalTarget = lastEvent.getAttribute("originalTarget");
      if (originalTarget.indexOf("label") != -1 || originalTarget.indexOf("tree") != -1)
      {
        eventlogger.removeEvent(lastEvent);
        return true;
      }
      else
        return false;
    }
    
    if (type == "command" && lastType == "click" && similarTime)
    {
      eventlogger.removeEvent(lastEvent);
      return true;
    }
    
    if (type == "click" && lastEvent && lastType == "unload" && similarTime)
      return false;
    
    if (type == "click" && similarTime && (lastType == "select" || lastType == "command"))
      return false;
    
    if (type == "dblclick")
    {
      eventlogger.removeEvent(lastEvent);
      lastEvent = eventlogger.eventList.lastChild;
      eventlogger.removeEvent(lastEvent);
      return true;
    }
    
    return true;
  },
  
  timeStampSimilar: function (aCurrTime, aPrevTime)
  {
    aCurrTime = parseInt(aCurrTime);
    aPrevTime = parseInt(aPrevTime);
    if ((aCurrTime - aPrevTime) < 300)
      return true;
    else
      return false;
  },
  
  removeEvent: function (aEventElement, aIndex, aFromFile)
  {
    if (aIndex && aEventElement.getAttribute("changePoint"))
    {
      var nextNode = eventlogger.eventList.childNodes[aIndex+1];
      if (nextNode && !nextNode.getAttribute("changePoint"))
        nextNode.setAttribute("changePoint", true);
    }
    
    eventlogger.eventList.removeChild(aEventElement);
    eventloggerUtils.outputEventList(eventlogger.eventDoc, true);
    
    // don't update the currentTarget array if it the event
    // came from a loaded file
    if (aFromFile)
      return;
    
    // if the call came from the Event Logger window, don't notify it
    if (aIndex)
    {
      eventlogger.currentTargets.splice(aIndex,1);
    }
    else
    {
      eventloggerUtils.notifyObservers("removeEvent",null);
      eventlogger.currentTargets.pop();
    }
  },
  
  // turns an event into XML, appends to the event tree
  appendEvent: function (aEvent)
  {
    var stringBundle = eventlogger.strings;
    var newEvent = eventlogger.eventDoc.createElement("event");

    if (aEvent.type == "type")
      newEvent.setAttribute("string", eventlogger.keypressStack);

    /*
      XXX workaround for bug 208427
    */
    var explicitOT = aEvent.explicitOriginalTarget.localName;
    var eventOT = aEvent.originalTarget.localName;
    if (eventOT == "div" && (explicitOT == "input" || explicitOT == "searchbar" || explicitOT == "textbox"))
      var realTarget = aEvent.explicitOriginalTarget;
    
    for (var n in aEvent)
    {
      if (aEvent[n] && !(aEvent[n] instanceof Function || n.toUpperCase() == n))
      {
        if (aEvent[n].toString().toLowerCase().indexOf("window") != -1)
        {
          var windowAttribs = eventlogger.getUniqueWindow(aEvent[n]);
          newEvent.setAttribute(n, windowAttribs);
        }
        else if (aEvent[n].toString().indexOf("object") != -1)
        {
          var objAttribs;
          if (realTarget != undefined && (n == "target" || n == "originalTarget" || n == "rangeParent"))
            objAttribs = eventlogger.getUniqueIdentifier(realTarget);
          else
            objAttribs = eventlogger.getUniqueIdentifier(aEvent[n]);
          newEvent.setAttribute(n, objAttribs);
        }
        else
          newEvent.setAttribute(n, aEvent[n]);
      }
    }
    
    var now = new Date();
    var currTime = now.getTime();
    newEvent.setAttribute("timeStamp",currTime);

    var eventDescription = eventlogger.getString(aEvent);
    newEvent.setAttribute("eventDescription", eventDescription);
    
    // check for a window change point
    if (aEvent.type != "unload" && aEvent.type != "resize" && aEvent.currentTarget
    && eventlogger.eventsFromDifferentWindows(aEvent))
      newEvent.setAttribute("changePoint",true);

    eventDescription = eventlogger.getLowerCaseEndings(eventDescription);
    var windowString = eventlogger.getWindowString(aEvent.currentTarget);
    var changeDescription = stringBundle.getFormattedString("windowchange",[windowString,eventDescription]);
    newEvent.setAttribute("changeDescription", changeDescription);
    
    // add to the event-list in memoery
    eventlogger.eventList.appendChild(newEvent);
    
    // dump XML to disk and notify observers (i.e. event logger console window)
    eventloggerUtils.outputEventList(eventlogger.eventDoc, true);
    var serializer = new XMLSerializer();
    var eventXML = serializer.serializeToString(newEvent);
    eventloggerUtils.notifyObservers("appendEvent",eventXML);
    
    // note the currentTarget for comparing to the next event
    eventlogger.currentTargets.push(aEvent.currentTarget);
  },
  
  // catch-all event handler
  handleEvent: function (aEvent)
  {
    if (aEvent.type == "resize" && aEvent.currentTarget.toString().toLowerCase().indexOf("window") != -1)
      return false;
  
    // decides whether to parse as keypress or "typing"
    if (aEvent.type == "keypress" && !eventlogger.checkKeypress(aEvent))
      eventlogger.appendTyping(aEvent);
    else
    {
      var eventWanted = eventlogger.filterEvent(aEvent)
      // ensure that typing is dispatched before other events to preserve chronology
      if (aEvent.type != "type" && eventWanted)
      {
        window.clearTimeout(eventlogger.keypressTimeout);
        if (eventlogger.keypressStack.length > 0)
          eventlogger.dispatchTyping();
      }
      if (eventWanted)    
        eventlogger.appendEvent(aEvent);
    }
  },
  
  // establishes if this is a command-based keypress or typing
  checkKeypress: function (aEvent)
  {
    if (aEvent.ctrlKey || aEvent.metaKey || aEvent.altKey)
      return true;
      
    if (aEvent.charCode == "")
      return true;
    
    return false;
  },

  appendTyping: function (aEvent)
  {
    var delay = eventlogger.appPrefs.getIntPref("keypressTimeout");
    window.clearTimeout(eventlogger.keypressTimeout);
    eventlogger.keypressTarget = aEvent.target;
    eventlogger.keypressStack += String.fromCharCode(aEvent.charCode);
    eventlogger.keypressTimeout = setTimeout("eventlogger.dispatchTyping()",delay);
  },
  
  dispatchTyping: function ()
  {
    var target = eventlogger.keypressTarget;
    var newEvent = target.ownerDocument.createEvent("Events");
    newEvent.initEvent("type",true,true);
    target.dispatchEvent(newEvent);
  },

  /*
    XXX areas for improvement:
    1) should check uniqueness 
    2) needs to cover more attributes
  */
  
  getUniqueIdentifier: function (aTarget)
  {
    // this avoids returning pseudo-elements
    while (aTarget.prefix)
      aTarget = aTarget.parentNode;
    
    if (aTarget.id != null && aTarget.id != "")
      return ["id", aTarget.id];
    
    if (aTarget.label != null & aTarget.label != "")
      return ["label", aTarget.label];
    
    if (aTarget.title != null && aTarget.title != "")
      return ["title", aTarget.title];
        
    if (aTarget.tooltipText != null && aTarget.tooltipText != "")
      return ["tooltipText", aTarget.tooltipText];

    if (aTarget.localName)
    {
      var localName = aTarget.localName;
      var tags = aTarget.ownerDocument.getElementsByTagName(localName);

      for (var index in tags)
      {
        if (tags[index] == aTarget)
          return ["localName", localName, index];
      }
    }
    else if (aTarget.tagName)
    {
      var tagName = aTarget.tagName;
      var tags = aTarget.ownerDocument.getElementsByTagName(tagName);

      for (var index in tags)
      {
        if (tags[index] == aTarget)
          return ["tagName", tagName, index];
      }
    }
    
    /*
      XXX might not be unique, but it'll have to do for now
    */
    
    if (aTarget.location)
    {
      return ["location", aTarget.location];
    }
    
    if (aTarget instanceof Text)
      return aTarget.nodeValue;
    
    dump("*** getUniqueIdentifier failed for " + aTarget + "\n");
    return false;
  },
  
  /*
    XXX needs to differentiate non-chrome windows somehow, is currently ignoring them
  */
  
  getUniqueWindow: function (aTarget)
  {
    var unique = true;
    var index = 0;
    var count = 1;
    var num = 0;
    var windows = eventlogger.windows;
      
    for (var n in windows)
    {
      if (aTarget.toString() == "[object ChromeWindow]")
      {
        if (windows[n] == aTarget)
        {
          index = n;
          num = count;
        }
        else
        {
          if (windows[n].location.toString() == aTarget.location.toString())
          {
            unique = false;
            count++;
          }
        }
      }
    }

    if (!unique)
      return [aTarget.location, index, num];
    else
      return aTarget.location;
  },
  
  // compiles a useful text string to describe an event
  getString: function (aEvent)
  {
    var stringBundle = eventlogger.strings;
    var originalTarget = null;
    var finalString = "";
    var returnString = "";
    var localName = aEvent.originalTarget.localName;
    var type = aEvent.type;
    var explicitOT = aEvent.explicitOriginalTarget.localName;

    if (localName && localName == "command" || (localName == "div" && (explicitOT == "input" || explicitOT == "searchbar" || explicitOT == "textbox")))
      originalTarget = eventlogger.getObjectString(aEvent.explicitOriginalTarget, aEvent.type);
    else
      originalTarget = eventlogger.getObjectString(aEvent.originalTarget, aEvent.type);

    var button;
    if (aEvent.button != undefined && aEvent.button < 3)
      button = stringBundle.getString("button" + aEvent.button);

    var relatedTarget = undefined;    
    if (aEvent.relatedTarget != undefined)
      relatedTarget = eventlogger.getObjectString(aEvent.relatedTarget, aEvent.type);

    if (localName && localName.indexOf("button") != -1)
    {
      originalTarget = stringBundle.getFormattedString("buttonevent",[originalTarget]);
      if (relatedTarget != undefined)
        relatedTarget = stringBundle.getFormattedString("buttonevent",[relatedTarget]);
    }

    if (localName && localName == "menu")
    {
      originalTarget = stringBundle.getFormattedString("menuevent",[originalTarget]);
      if (relatedTarget != undefined)
        relatedTarget = stringBundle.getFormattedString("menuevent",[relatedTarget]);
    }
    
    switch (type)
    {
      case "click":
      {
        if (localName.indexOf("button") != -1 && localName.indexOf("toolbar") == -1)
          finalString = stringBundle.getFormattedString("buttonclick",[originalTarget]);
        else
          finalString = stringBundle.getFormattedString(type,[button,originalTarget]);
        break;
      }
      case "mousedown":
      case "mouseup":
      {
        finalString = stringBundle.getFormattedString(type,[button.toLowerCase(),originalTarget]);
        break;
      }
      case "select":
      {
        if (aEvent.originalTarget.selectedItem)
          element = aEvent.originalTarget.selectedItem.localName;
        else
          element = aEvent.originalTarget.localName;
        
        element = stringBundle.getString(element);
        finalString = stringBundle.getFormattedString(type,[originalTarget, element]);
        break;
      }
      case "type":
      {
        var string = eventlogger.keypressStack;
        eventlogger.keypressStack = "";
        finalString = stringBundle.getFormattedString(type,[string]);
        break;
      }
      case "keypress":
      {
        var key;
        if (aEvent.keyCode != 0)
          key = stringBundle.getString("key" + aEvent.keyCode);
        else
          key = String.fromCharCode(aEvent.charCode).toUpperCase();
        finalString = stringBundle.getFormattedString(type,[key]);
        break;
      }
      case "resize":
      case "unload":
      {
        var windowString = eventlogger.getWindowString(aEvent.currentTarget);
        finalString = stringBundle.getFormattedString(type,[windowString]);
        break;
      }
      case "command":
      {
        if (localName == "checkbox" || (localName == "input" && aEvent.originalTarget.type == "checkbox"))
        {
          type = (aEvent.originalTarget.checked) ? "checkboxtick" : "checkboxuntick";
          originalTarget = aEvent.originalTarget.label;
        }
        
        if (localName && localName.indexOf("button") != -1)
          type = "button" + type;
        finalString = stringBundle.getFormattedString(type,[originalTarget]);
        break;
      }
      default:
      {
        /*
          XXX once this code is good enough, remove excess braces and
          *** notification that default case was reached
        */
        finalString = stringBundle.getFormattedString(type,[originalTarget]);
      }
    }
    
    var modString = eventlogger.getModifierString(eventlogger.getModifierArray(aEvent));
    if (modString != null)
    {
      finalString = eventlogger.getLowerCaseEndings(finalString);
      returnString = stringBundle.getFormattedString("modifierstring",[modString,finalString]);
    }
    else
      returnString = finalString;
      
    return returnString;
  },

  /*
    XXX this really needs to be better, but essentially works by lowercasing the first and last
    character of the badly named finalString before adding modifier key data to it.
  */
  getLowerCaseEndings: function (aString)
  {
    aString = aString[0].toLowerCase() + aString.substring(1,aString.length);
    aString = aString.substring(0,aString.length-1) + aString[aString.length-1].toLowerCase();
    return aString;
  },
  
  // gets a string to describe the currentTarget window
  getWindowString: function (aTarget)
  {
    var stringBundle = eventlogger.strings;
    var windowAttribs = eventlogger.getUniqueWindow(aTarget);
    var title = aTarget.document.title;
    
    if (windowAttribs instanceof Array)
      var num = windowAttribs[2];
    
    if (num != undefined)
      returnString = stringBundle.getFormattedString("windowmulti",[title,num]);
    else
      returnString = stringBundle.getFormattedString("window",[title]);
    
    return returnString;
  },
  
  // returns a string if the window changed between the last and current event
  eventsFromDifferentWindows: function (aEvent)
  {
    var lastCurrentTarget = eventlogger.currentTargets[eventlogger.currentTargets.length-1];
    if (!lastCurrentTarget || !aEvent.currentTarget)
      return false;
      
    return (aEvent.currentTarget != lastCurrentTarget);
  },
  
  getModifierString: function (aMods)
  {
    if (aMods.length > 0)
    {
      var stringBundle = eventlogger.strings;
      var numMods = "modifier" + aMods.length;
      return stringBundle.getFormattedString(numMods,aMods);
    }
  },
  
  getModifierArray: function (aEvent)
  {
    var stringBundle = eventlogger.strings;
    var modifierKeys = new Array();
    
    if (aEvent.ctrlKey)
      modifierKeys.push(stringBundle.getString("ctrl"));
    
    if (aEvent.shiftKey)
      modifierKeys.push(stringBundle.getString("shift"));
    
    if (aEvent.altKey)
      modifierKeys.push(stringBundle.getString("alt"));
      
    if (aEvent.metaKey)
      modifierKeys.push(stringBundle.getString("meta"));
    
    return modifierKeys;
  },
  
  // extract a useful string to describe the target object
  getObjectString: function (aTarget, aType)
  {
    var stringBundle = eventlogger.strings;   
    var localName = null;
    var nodeName = null;
    var unique = true;
    var finalString = "";
    var prefix = null;
    var string = null;
    var suffix = null;
    
    if (aTarget.localName)
      localName = aTarget.localName.toLowerCase();
    
    if (aTarget.nodeName)
      nodeName = aTarget.nodeName.toLowerCase();
      
    if (!localName && !nodeName)
    {
      dump("*** getObjectString() failed for " + aTarget + "\n");
      return "object";
    }

    if (nodeName && nodeName == "#document")
    {
      if (aTarget.location.protocol == "chrome:")
        return aTarget.documentElement.title;
      else
        return aTarget.defaultView.document.title;
    }
    
    if (localName == "input")
    {
      localName += aTarget.type;
    }
    
    /*
      XXX workaround for neither the close-tab button or the close-findbar buttons having labels
    */
    if (localName == "toolbarbutton" && aTarget.className.indexOf("tabs-closebutton") != -1)
      return stringBundle.getString("closetab");
    
    if (localName == "toolbarbutton" && aTarget.id == "find-closebutton")
      return stringBundle.getString("closefindbar");
    
    if (localName == "textbox" && aTarget.id && aTarget.id == "urlbar")
      return stringBundle.getString("urlbar");      
    
    if (localName == "searchbar")
      return stringBundle.getString(localName);

    if (localName.indexOf("button") != -1 || localName == "menuitem" || localName == "menu" || localName == "command")
      return aTarget.getAttribute("label");
    
    if (aType == "select")
    {
      if (localName == "listbox" || localName == "tabs" || localName == "radiogroup")
        return aTarget.selectedItem.label;
    }

    if (localName)
    {
      var tags = aTarget.ownerDocument.getElementsByTagName(localName);

      for (var index in tags)
      {
        if (tags[index] == aTarget)
        {
          unique = false;
          break;
        }
      }
    }
    
    if (!unique)
    {
      // change from array element to more human-friendly number
      index++;
      
      prefix = stringBundle.getFormattedString("multiprefix",[index]).replace(/_/g," ");
      string = stringBundle.getString(localName);
      suffix = stringBundle.getFormattedString("multisuffix",[index]).replace(/_/g," ");;
    }
    else
    {
      prefix = stringBundle.getString("singleprefix").replace(/_/g," ");
      string = stringBundle.getString(localName);
      suffix = stringBundle.getString("singlesuffix").replace(/_/g," ");
    }
    
    if (prefix != null)
      finalString += prefix;
      
    finalString += string;
    
    if (suffix != null)
      finalString += suffix;
  
    return finalString;
  },

};

var eventloggerPrefObserver =
{
  register: function()
  {
    if (!eventlogger.eventPrefs || !eventlogger.appPrefs)
      return;

    var appPrefs = eventlogger.appPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    appPrefs.addObserver("", this, false);
    
    var eventPrefs = eventlogger.eventPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    eventPrefs.addObserver("", this, false);
  },

  unregister: function()
  {
    var appPrefs = eventlogger.appPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    appPrefs.removeObserver("", this, false);
    
    var eventPrefs = eventlogger.eventPrefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    eventPrefs.removeObserver("", this, false);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic != "nsPref:changed")
      return;
      
    if (aData == "logging")
    {
      if (eventlogger.appPrefs.getBoolPref(aData))
        eventlogger.startLogging();
      else
        eventlogger.stopLogging();
      return;
    }
    
    aData = aData.substring(7, aData.length);
    var addEvent = eventlogger.eventPrefs.getBoolPref(aData);
    var logging = eventlogger.appPrefs.getBoolPref("logging");
    
    for (var n in eventlogger.windows)
    {
      if (addEvent && logging)
        eventlogger.addEventListener(eventlogger.windows[n], aData);
      else
        eventlogger.removeEventListener(eventlogger.windows[n], aData);
    }
  }
};

var eventloggerWindowObserver =
{
  register: function()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.addObserver(this, "domwindowopened", false);
    os.addObserver(this, "domwindowclosed", false);
  },

  unregister: function ()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.removeObserver(this, "domwindowopened", false);
    os.removeObserver(this, "domwindowclosed", false);
  },
  
  observe: function (aSubject, aTopic, aData)
  {
    if (aTopic == "domwindowopened")
    {
      /*
        XXX workaround for bug 197709
      */
      var now = new Date();
      eventlogger.lastWindowOpen = now.getTime();

      eventlogger.windows.push(aSubject);
      if (eventlogger.appPrefs.getBoolPref("logging"))
        eventlogger.addEventListeners(aSubject);
    }
    else
    {
      var index = eventlogger.windows.indexOf(aSubject);
      eventlogger.windows.splice(index,1);
    }
  }
};

var eventloggerDeleteObserver =
{
  register: function()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.addObserver(this, "deleteEvent", false);
  },

  unregister: function ()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.removeObserver(this, "deleteEvent", false);
  },
  
  observe: function (aSubject, aTopic, aData)
  {
    var index = aData[0];
    var fromFile = aData[1];
    var eventNode = eventlogger.eventList.childNodes[index];
    eventlogger.removeEvent(eventNode, index, fromFile);
  }
};

var eventloggerFileChangeObserver =
{
  register: function()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.addObserver(this, "loadFile", false);
  },

  unregister: function ()
  {
    var os = Components.classes["@mozilla.org/observer-service;1"]
               .getService(Components.interfaces.nsIObserverService);
               
    os.removeObserver(this, "loadFile", false);
  },
  
  observe: function (aSubject, aTopic, aData)
  {
    eventlogger.loadTemp();
  }
};

window.addEventListener("load", eventlogger.init, false);